////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Sample application using my compression/decompression class
//
//  Files: sample.cpp
//
//  By Rasmus Neckelmann (mullen@mailme.dk), 27/6-2001
// 

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "cmpr.h"

void main(int nNumArguments,char **ppcArguments) {
	int i;
	FILE *fp;
	int nInputDataSize;
	void *pvInputData;
	
	CCmpr *pCmpr;

	char *pcInputFile=NULL;
	char *pcOutputFile=NULL;
	bool bDecompress=FALSE;

	// Parse command-line arguments //
	for(i=1;i<nNumArguments;i++) {
		if(!strcmp(ppcArguments[i],"-d") || !strcmp(ppcArguments[i],"-D")) {
			bDecompress=TRUE;
		}
		else if(!strcmp(ppcArguments[i],"-c") || !strcmp(ppcArguments[i],"-C")) {
			bDecompress=FALSE;
		}
		else {
			if(pcInputFile==NULL) pcInputFile=ppcArguments[i];
			else pcOutputFile=ppcArguments[i];
		}
	}

	// Should we display some help text? //
	if(pcInputFile==NULL || pcOutputFile==NULL) {
		printf("usage: sample infile outfile [options]\n");
		printf("options: (* = default)\n");
		printf("  -d    Decompress\n");
		printf("* -c    Compress\n");
		return;
	}

	// Create compression/decompression object //
	pCmpr=new CCmpr;

	// Read input file //
	fp=fopen(pcInputFile,"rb");
	if(!fp) {
		printf("Could not open input file: %s!\n",pcInputFile);
		return;
	}
	
	fseek(fp,0,SEEK_END);
	nInputDataSize=ftell(fp);
	fseek(fp,0,SEEK_SET);

	pvInputData=(void *)malloc(nInputDataSize);
	fread(pvInputData,nInputDataSize,1,fp);
	fclose(fp);

	// Set up compressor/decompressor //
	pCmpr->SetInputData(pvInputData,nInputDataSize);

	// Compress/decompress //
	if(bDecompress) {
		printf("Decompressing data..."); fflush(stdout);
		if(!pCmpr->Uncompress()) {
			printf("Something went wrong while decompressing data!\n");
			return;
		}			
		printf("OK\n");
	}
	else {
		printf("Compressing data..."); fflush(stdout);
		if(!pCmpr->Compress()) {
			printf("Something went wrong while compressing data!\n");
			return;
		}			
		printf("OK\n");	
		printf("%d -> %d (%d%c)\n",nInputDataSize,pCmpr->GetOutputSize(),(pCmpr->GetOutputSize()*100)/nInputDataSize,'%');
	}

	// Save //
	fp=fopen(pcOutputFile,"wb");
	if(!fp) {
		printf("Could not open output file: %s!\n",pcOutputFile);
		return;
	}

	fwrite(pCmpr->GetOutputData(),pCmpr->GetOutputSize(),1,fp);
	fclose(fp);

	// Clean up //
	free(pCmpr->GetOutputData());
	delete pCmpr;
}
